<?php

namespace SGalinski\SgNews\Command;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgNews\Domain\Model\News;
use TYPO3\CMS\Backend\FrontendBackendUserAuthentication;
use TYPO3\CMS\Core\Database\DatabaseConnection;
use TYPO3\CMS\Core\DataHandling\DataHandler;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\CommandController;
use TYPO3\CMS\Extbase\Persistence\Exception\IllegalObjectTypeException;
use TYPO3\CMS\Extbase\Persistence\Exception\UnknownObjectException;

/**
 * Command controller, that migrates data from tx_news to sg_news
 */
class MigrateNewsCommandController extends CommandController {

	/**
	 * @var bool
	 */
	protected $requestAdminPermissions = TRUE;

	/**
	 * @var \TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager
	 * @inject
	 */
	protected $persistenceManager;

	/**
	 * @var \SGalinski\SgNews\Domain\Repository\NewsRepository
	 * @inject
	 */
	protected $newsRepository;

	/**
	 * this array maps new pages to their original entry in the tx_news table
	 *
	 * @var array $newsPagesMap
	 */
	protected $newsPagesMap = [];

	/**
	 * this array contains all pages that should be migrated to sg_news
	 *
	 * @var array $pagesToMigrate
	 */
	protected $pagesToMigrate = [];

	/**
	 * @var array $languageMap
	 */
	protected $languageMap = [];

	/**
	 * @param string $copyPageId the page id of the template that should be copied
	 * @param int $categoryPid the page id of the category page
	 * @param int $year only news from that year will be migrated
	 * @param string $languageMapAsJson a json, mapping language ids (old => new). this is needed if the sys_language_uids have changed
	 *
	 * @throws IllegalObjectTypeException
	 * @throws UnknownObjectException
	 */
	public function runMigrateNewsCommand(
		$copyPageId, $categoryPid, $year = 2015,
		$languageMapAsJson = '{"3":1,"1":0,"2":2,"0":3}'
	) {
		$this->languageMap = json_decode($languageMapAsJson, TRUE);

		/** @var DatabaseConnection $db */
		$db = $GLOBALS['TYPO3_DB'];
		$where = 'hidden = 0 and deleted = 0';
		/** @var \mysqli_result $result */
		$result = $db->exec_SELECTquery('*', 'tx_news_domain_model_news', $where);

		$localDataHandler = GeneralUtility::makeInstance(DataHandler::class);
		$beUser = $this->simulateBackendUser();
		$localCommandMap = [
			'pages' => [
				$copyPageId => [
					'copy' => (int) $categoryPid
				]
			]
		];

		$resultArray = [];
		while ($row = $result->fetch_assoc()) {
			// ignore the entry if its not within the given year
			if ((int) date('Y', ($row['datetime'])) !== $year) {
				continue;
			}

			$resultArray[] = $row;

			// if no l10n_parent exists, create a copy of the page
			if ((int) $row['l10n_parent'] === 0) {
				$localDataHandler->start([], $localCommandMap, $beUser);
				$localDataHandler->bypassAccessCheckForRecords = TRUE;
				$localDataHandler->checkModifyAccessList('pages');
				$localDataHandler->process_cmdmap();

				// get the id of the new object
				$newPageId = $localDataHandler->copyMappingArray['pages'][$copyPageId];

				// make entry in news map, so we can fetch it later
				$this->newsPagesMap[$row['uid']] = $newPageId;

				/** @var News $newsPage */
				$newsPage = $this->newsRepository->findByUidIgnoreEnableFields($newPageId);

				if ($newsPage !== NULL) {
					// @todo create tt content element, map images, author, categories etc.
					$title = date('Y-m-d', $row['datetime']) . ' - ' . $row['title'];
					$newsPage->setTitle($title);
					$newsPage->setAuthor($row['author']);

					$this->newsRepository->update($newsPage);
					$this->persistenceManager->persistAll();

					// update content element from the new page
					$where = 'pid = ' . $newPageId . ' AND sys_language_uid = ' . (int) $row['sys_language_uid'];
					$db->exec_UPDATEquery('tt_content', $where, ['bodytext' => $row['bodytext']]);
				}
			} else { // this row is a translation and should simply update the content accordingly
				$this->updateTranslation($row);
			}
		}

		$this->pagesToMigrate = $resultArray;
	}

	/**
	 * Updates content element of a news translationuid
	 *
	 * @param array $row
	 */
	private function updateTranslation(array $row) {
		/** @var DatabaseConnection $db */
		$db = $GLOBALS['TYPO3_DB'];

		// get entry in news map
		$parentId = $this->newsPagesMap[$row['l10n_parent']];

		// get content element from the original page
		$where = 'pid = ' . (int) $parentId;

		/** @var \mysqli_result $result */
		$result = $db->exec_SELECTquery('l18n_parent', 'tt_content', $where);
		$originalContentElement = $result->fetch_row();

		$where = 'l18n_parent = ' . (int) $originalContentElement[0];

		// look up the correct language id, if they have changed
		if ($this->languageMap[(int) $row['sys_language_uid']]) {
			$where .= ' AND sys_language_uid = ' . $this->languageMap[(int) $row['sys_language_uid']];
		} else {
			$where .= ' AND sys_language_uid = ' . (int) $row['sys_language_uid'];
		}
		$db->exec_UPDATEquery('tt_content', $where, ['bodytext' => $row['bodytext']]);
	}

	/**
	 * Simulate Backend User for DataHandler
	 *
	 * @return FrontendBackendUserAuthentication
	 */
	private function simulateBackendUser() {
		/** @var \TYPO3\CMS\Backend\FrontendBackendUserAuthentication $BE_USER */
		$BE_USER = GeneralUtility::makeInstance(FrontendBackendUserAuthentication::class);
		$BE_USER->setBeUserByName('admin');
		if ($BE_USER->user['uid']) {
			$BE_USER->fetchGroupData();
		}
		$BE_USER->uc_default['copyLevels'] = '9999';
		$BE_USER->uc = $BE_USER->uc_default;
		$GLOBALS['PAGES_TYPES'][254]['allowedTables'] = '*';
		return $BE_USER;
	}
}
